#include "TestMutex.h"

#include <OSAL/os_time.h>


void test_DelayParameterChange();

void test_DelayParameterUse();

void ctor_TestMutex(TestMutex* thiz) {
  thiz->datasum = 45;
  thiz->data1 = 34;
  thiz->data2 = thiz->datasum - thiz->data1;
  thiz->mutex = os_createMutex("mutex");
}



/**This routine is an example to calculate and store parameter.
* It can be called in the main loop or a lower priority thread.
* The parameter are used consistent, therefore a mutex while writing is necessary.
*/
void calculateParameter_TestMutex(TestMutex* thiz) {
  int16 x = thiz->x1;          //any input value
  if(++x > 100){ x = -100; }   //runs from -100 .. 100 
  thiz->x1 = x;

  os_lockMutex(thiz->mutex, 0);   //exclusively access starts:
  { thiz->bWrParam = 1;
    thiz->data1 = x;           //change first value
    test_DelayParameterChange();              //it is factitious, force higher possibility to interrupt
    thiz->data2 = thiz->datasum - x;  //should be consistent.
    thiz->bWrParam = 0;
    os_unlockMutex(thiz->mutex); //exclusive access ends.
  }
  thiz->ctWriteParem +=1;
}




void workThread_TestMutex(TestMutex* thiz) {
  STACKTRC_ENTRY("workThread");
  TRY{
    useParameter_TestMutex(thiz);
  }_TRY
  CATCH(Exception, exc) {
    unused_emC(exc);
    thiz->ctCatched +=1;
  }
  END_TRY
  STACKTRC_LEAVE;
}






/**This routine is called either in a second thread or in interrupt.
* It uses the consistent parameter. To prevent a thread switch to the thread
* which changes the parameter while accessing, a mutex is used.
*/
void useParameter_TestMutex(TestMutex* thiz) {
  STACKTRC_ENTRY("useParameter_TestMutex");
  int16 val1, val2, sum;            //use local copy to work
  thiz->ctUsage +=1;
  if (thiz->bWrParam) {
    thiz->ctIntrWrParam +=1;
  }

  os_lockMutex(thiz->mutex, 0);  //exclusive access starts
  { val1 = thiz->data1;          //access the first value.
    test_DelayParameterUse();    //it is factitious, force higher possibility to interrupt
    val2 = thiz->data2;          //access the 2. value, interruptable.
    sum = thiz->datasum;
    os_unlockMutex(thiz->mutex); //exclusive access ends.
  }
   
  if (val1 + val2 != sum) {      //use the paramteter values, here check consistentcy
    thiz->ctError += 1;
    THROW_s0(IllegalArgumentException, "parameter inconsistently", sum, val1);
  }
  else {
    thiz->ctOk += 1;
  }
  thiz->ctUsageOk +=1;
  int test = getMaxStackDepth_ThreadContext_emC(_thCxt);
  STACKTRC_LEAVE;
}


