#include "TestNumericSimple.h"

#include <OSAL/os_time.h>

#ifndef __DONOTUSE_REFLECTION__
  #define __Use_Inspector__
  #include <Inspc/Service_Inspc.h>
#endif

#include <stdio.h>


#ifdef __DONOTUSE_REFLECTION__
  char const reflection_MyData[] = "REFLMyData";
#else
  #include "../genRefl/TestNumericSimple.crefl"
#endif

//Hint: CONST_MyData is a define which follows with { { ....} ...} the typedef of Mydata.
//The using of the macor of user level should present only the important things.
MyData data = INIZ_MyData(data, &reflection_MyData);


#ifdef __Use_Inspector__
//The inspector service, it is a part of the runtime environment.
Inspector_Inspc_s theInspector = { 0 };
#endif //__Use_Inspector__

int main(int nArgs, char** sArgs) {
  STACKTRC_ROOT_ENTRY("main");
  ctor_MyData(&data);
#ifdef __Use_Inspector__
  ctorO_Inspector_Inspc(&theInspector.base.object, s0_StringJc("UDP:0.0.0.0:60094"), _thCxt);
  start_Inspector_Inspc_F(&theInspector, &data.base.object, _thCxt);
#endif //__Use_Inspector__
  data.base.super.bRun = 1;

  TRY{
    test_MyData(&data, 124.7f); //forces a null-pointer exception in C++
  }_TRY
    CATCH(Exception, exc) {
    printStackTrace_ExceptionJc(exc, _thCxt);
    data.testThrowResult = 0;  //falback strategy: This calculation may faulty.
  }END_TRY;
  //
  TRY{
    testTry(&data);
  }_TRY
    CATCH(Exception, exc) {
    printStackTrace_ExceptionJc(exc, _thCxt);
    data.testThrowResult = 0;  //falback strategy: This calculation may faulty.
  }END_TRY;

  calculateInLoop(&data);    //to test reflection access via inspector.
  STACKTRC_LEAVE; return 0;
}


MyData* ctor_MyData(MyData* thiz) {
  initReflection_ObjectJc(&thiz->base.object, thiz, sizeof(MyData), &reflection_MyData, 0);
  return thiz;
}



void calculateInLoop(MyData* thiz) {
  thiz->a = 5.1f;   //exampledata
  thiz->b = 7.5f;  
  while (thiz->base.super.bRun) {
    thiz->y = thiz->a * thiz->b;
    os_delayThread(1);
  }

}


void test_MyData(MyData* thiz, float val){
  STACKTRC_ENTRY("test_MyData");
  thiz->array[0] = val;
  float* array1 = thiz->array; //not recommended, size is unknown.
  CALLINE;
  #ifdef __cplusplus
    array1 = null;               //faulty, forces asynchron exception, works only in C++
  #endif
  array1[1] = 5.0f * val;
  STACKTRC_LEAVE;
}



/**This routine contains a TRY...CATCH and continues work. */
float testTry(MyData* thiz) {
  STACKTRC_ENTRY("testTry");
  float val;
  TRY{
    val = testTryLevel2(thiz, _thCxt);
  }_TRY
  CATCH(ClassCastException, exc) {
    //It is not recommended to do printStacktrace in normal case
    //because the exception is handled here. Only for debug, it may be important.
    printStackTrace_ExceptionJc(exc, _thCxt);
    //But save a log entry may be proper if the exception is not expected:
    val = 0;
  }FINALLY{
    val = 1;
  }END_TRY;
  STACKTRC_LEAVE;
  return val;
}



/**It is a called routine from TRY level without catch.*/
float testTryLevel2(MyData* thiz, ThCxt* _thCxt) {
  STACKTRC_TENTRY("testTryLevel2");
  CALLINE; float val = testThrow(thiz, 10, 2.0f, _thCxt);
  STACKTRC_LEAVE;
  return val;
}



float testThrow(MyData* thiz, int ix, float val, ThCxt* _thCxt) {
  STACKTRC_TENTRY("testThrow");
  int stackSizeMax = getMaxStackDepth_ThreadContext_emC(_thCxt); //only for debug
  if (ix < 0 || ix >= ARRAYLEN_emC(thiz->array)) {
    char msg[40] = {0};
    snprintf(msg, sizeof(msg), "faulty index:%d for value %f", ix, val);
    thiz->testThrowResult = -1;  //to document: invalid.
    THROW1_s0(IndexOutOfBoundsException, msg, ix);
    ix = ARRAYLEN_emC(thiz->array) -1;  //replacement on error: Set index to the last element.
    stackSizeMax = getMaxStackDepth_ThreadContext_emC(_thCxt); //only for debug
  }
  thiz->array[ix] = val;
  { int ix;
    thiz->testThrowResult = 0;
    for (ix = 0; ix < ARRAYLEN_emC(thiz->array); ++ix) {
      thiz->testThrowResult += thiz->array[ix]; 
    }
  }
  STACKTRC_LEAVE; return thiz->array[0];
}


